@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 //go:build !js
 // +build !js
 
@@ -12,11 +15,11 @@ import (
 	"time"
 
 	"github.com/pion/logging"
-	"github.com/pion/stun"
-	"github.com/pion/transport/v2/test"
-	"github.com/pion/transport/v2/vnet"
-	"github.com/pion/turn/v2"
-	"github.com/stretchr/testify/assert"
+	"github.com/pion/stun/v3"
+	"github.com/pion/transport/v3/test"
+	"github.com/pion/transport/v3/vnet"
+	"github.com/pion/turn/v4"
+	"github.com/stretchr/testify/require"
 )
 
 const (
@@ -38,11 +41,11 @@ type virtualNet struct {
 }
 
 func (v *virtualNet) close() {
-	v.server.Close() // nolint:errcheck,gosec
-	v.wan.Stop()     // nolint:errcheck,gosec
+	v.server.Close() //nolint:errcheck,gosec
+	v.wan.Stop()     //nolint:errcheck,gosec
 }
 
-func buildVNet(natType0, natType1 *vnet.NATType) (*virtualNet, error) {
+func buildVNet(natType0, natType1 *vnet.NATType) (*virtualNet, error) { //nolint:cyclop
 	loggerFactory := logging.NewDefaultLoggerFactory()
 
 	// WAN
@@ -55,7 +58,7 @@ func buildVNet(natType0, natType1 *vnet.NATType) (*virtualNet, error) {
 	}
 
 	wanNet, err := vnet.NewNet(&vnet.NetConfig{
-		StaticIP: vnetSTUNServerIP, // will be assigned to eth0
+		StaticIP: vnetSTUNServerIP, // Will be assigned to eth0
 	})
 	if err != nil {
 		return nil, err
@@ -74,6 +77,7 @@ func buildVNet(natType0, natType1 *vnet.NATType) (*virtualNet, error) {
 					vnetGlobalIPA + "/" + vnetLocalIPA,
 				}
 			}
+
 			return []string{
 				vnetGlobalIPA,
 			}
@@ -111,6 +115,7 @@ func buildVNet(natType0, natType1 *vnet.NATType) (*virtualNet, error) {
 					vnetGlobalIPB + "/" + vnetLocalIPB,
 				}
 			}
+
 			return []string{
 				vnetGlobalIPB,
 			}
@@ -168,10 +173,11 @@ func addVNetSTUN(wanNet *vnet.Net, loggerFactory logging.LoggerFactory) (*turn.S
 		return nil, err
 	}
 	server, err := turn.NewServer(turn.ServerConfig{
-		AuthHandler: func(username, realm string, srcAddr net.Addr) (key []byte, ok bool) {
+		AuthHandler: func(username, realm string, _ net.Addr) (key []byte, ok bool) {
 			if pw, ok := credMap[username]; ok {
 				return turn.GenerateAuthKey(username, realm, pw), true
 			}
+
 			return nil, false
 		},
 		PacketConnConfigs: []turn.PacketConnConfig{
@@ -194,15 +200,16 @@ func addVNetSTUN(wanNet *vnet.Net, loggerFactory logging.LoggerFactory) (*turn.S
 	return server, err
 }
 
-func connectWithVNet(aAgent, bAgent *Agent) (*Conn, *Conn) {
+func connectWithVNet(t *testing.T, aAgent, bAgent *Agent) (*Conn, *Conn) {
+	t.Helper()
 	// Manual signaling
 	aUfrag, aPwd, err := aAgent.GetLocalUserCredentials()
-	check(err)
+	require.NoError(t, err)
 
 	bUfrag, bPwd, err := bAgent.GetLocalUserCredentials()
-	check(err)
+	require.NoError(t, err)
 
-	gatherAndExchangeCandidates(aAgent, bAgent)
+	gatherAndExchangeCandidates(t, aAgent, bAgent)
 
 	accepted := make(chan struct{})
 	var aConn *Conn
@@ -210,24 +217,26 @@ func connectWithVNet(aAgent, bAgent *Agent) (*Conn, *Conn) {
 	go func() {
 		var acceptErr error
 		aConn, acceptErr = aAgent.Accept(context.TODO(), bUfrag, bPwd)
-		check(acceptErr)
+		require.NoError(t, acceptErr)
 		close(accepted)
 	}()
 
 	bConn, err := bAgent.Dial(context.TODO(), aUfrag, aPwd)
-	check(err)
+	require.NoError(t, err)
 
 	// Ensure accepted
 	<-accepted
+
 	return aConn, bConn
 }
 
 type agentTestConfig struct {
-	urls                   []*URL
+	urls                   []*stun.URI
 	nat1To1IPCandidateType CandidateType
 }
 
-func pipeWithVNet(v *virtualNet, a0TestConfig, a1TestConfig *agentTestConfig) (*Conn, *Conn) {
+func pipeWithVNet(t *testing.T, vnet *virtualNet, a0TestConfig, a1TestConfig *agentTestConfig) (*Conn, *Conn) {
+	t.Helper()
 	aNotifier, aConnected := onConnected()
 	bNotifier, bConnected := onConnected()
 
@@ -244,17 +253,12 @@ func pipeWithVNet(v *virtualNet, a0TestConfig, a1TestConfig *agentTestConfig) (*
 		MulticastDNSMode:       MulticastDNSModeDisabled,
 		NAT1To1IPs:             nat1To1IPs,
 		NAT1To1IPCandidateType: a0TestConfig.nat1To1IPCandidateType,
-		Net:                    v.net0,
+		Net:                    vnet.net0,
 	}
 
 	aAgent, err := NewAgent(cfg0)
-	if err != nil {
-		panic(err)
-	}
-	err = aAgent.OnConnectionStateChange(aNotifier)
-	if err != nil {
-		panic(err)
-	}
+	require.NoError(t, err)
+	require.NoError(t, aAgent.OnConnectionStateChange(aNotifier))
 
 	if a1TestConfig.nat1To1IPCandidateType != CandidateTypeUnspecified {
 		nat1To1IPs = []string{
@@ -267,19 +271,14 @@ func pipeWithVNet(v *virtualNet, a0TestConfig, a1TestConfig *agentTestConfig) (*
 		MulticastDNSMode:       MulticastDNSModeDisabled,
 		NAT1To1IPs:             nat1To1IPs,
 		NAT1To1IPCandidateType: a1TestConfig.nat1To1IPCandidateType,
-		Net:                    v.net1,
+		Net:                    vnet.net1,
 	}
 
 	bAgent, err := NewAgent(cfg1)
-	if err != nil {
-		panic(err)
-	}
-	err = bAgent.OnConnectionStateChange(bNotifier)
-	if err != nil {
-		panic(err)
-	}
+	require.NoError(t, err)
+	require.NoError(t, bAgent.OnConnectionStateChange(bNotifier))
 
-	aConn, bConn := connectWithVNet(aAgent, bAgent)
+	aConn, bConn := connectWithVNet(t, aAgent, bAgent)
 
 	// Ensure pair selected
 	// Note: this assumes ConnectionStateConnected is thrown after selecting the final pair
@@ -289,33 +288,61 @@ func pipeWithVNet(v *virtualNet, a0TestConfig, a1TestConfig *agentTestConfig) (*
 	return aConn, bConn
 }
 
-func closePipe(t *testing.T, ca *Conn, cb *Conn) bool {
-	err := ca.Close()
-	if !assert.NoError(t, err, "should succeed") {
-		return false
+func pipeWithVNetUsingOptions(t *testing.T, opts0, opts1 []AgentOption) (*Conn, *Conn) {
+	t.Helper()
+
+	aNotifier, aConnected := onConnected()
+	bNotifier, bConnected := onConnected()
+
+	aAgent, err := NewAgentWithOptions(opts0...)
+	require.NoError(t, err)
+	if err = aAgent.OnConnectionStateChange(aNotifier); err != nil {
+		require.NoError(t, err)
 	}
-	err = cb.Close()
-	return assert.NoError(t, err, "should succeed")
+
+	bAgent, err := NewAgentWithOptions(opts1...)
+	require.NoError(t, err)
+	if err = bAgent.OnConnectionStateChange(bNotifier); err != nil {
+		require.NoError(t, err)
+	}
+
+	t.Cleanup(func() {
+		require.NoError(t, aAgent.Close(), "failed to close agent0")
+		require.NoError(t, bAgent.Close(), "failed to close agent1")
+	})
+
+	aConn, bConn := connectWithVNet(t, aAgent, bAgent)
+
+	<-aConnected
+	<-bConnected
+
+	return aConn, bConn
+}
+
+func closePipe(t *testing.T, ca *Conn, cb *Conn) {
+	t.Helper()
+
+	require.NoError(t, ca.Close())
+	require.NoError(t, cb.Close())
 }
 
 func TestConnectivityVNet(t *testing.T) {
-	report := test.CheckRoutines(t)
-	defer report()
+	defer test.CheckRoutines(t)()
 
-	stunServerURL := &URL{
-		Scheme: SchemeTypeSTUN,
+	stunServerURL := &stun.URI{
+		Scheme: stun.SchemeTypeSTUN,
 		Host:   vnetSTUNServerIP,
 		Port:   vnetSTUNServerPort,
-		Proto:  ProtoTypeUDP,
+		Proto:  stun.ProtoTypeUDP,
 	}
 
-	turnServerURL := &URL{
-		Scheme:   SchemeTypeTURN,
+	turnServerURL := &stun.URI{
+		Scheme:   stun.SchemeTypeTURN,
 		Host:     vnetSTUNServerIP,
 		Port:     vnetSTUNServerPort,
 		Username: "user",
 		Password: "pass",
-		Proto:    ProtoTypeUDP,
+		Proto:    stun.ProtoTypeUDP,
 	}
 
 	t.Run("Full-cone NATs on both ends", func(t *testing.T) {
@@ -327,32 +354,28 @@ func TestConnectivityVNet(t *testing.T) {
 			MappingBehavior:   vnet.EndpointIndependent,
 			FilteringBehavior: vnet.EndpointIndependent,
 		}
-		v, err := buildVNet(natType, natType)
+		vnet, err := buildVNet(natType, natType)
 
-		if !assert.NoError(t, err, "should succeed") {
-			return
-		}
-		defer v.close()
+		require.NoError(t, err, "should succeed")
+		defer vnet.close()
 
 		log.Debug("Connecting...")
 		a0TestConfig := &agentTestConfig{
-			urls: []*URL{
+			urls: []*stun.URI{
 				stunServerURL,
 			},
 		}
 		a1TestConfig := &agentTestConfig{
-			urls: []*URL{
+			urls: []*stun.URI{
 				stunServerURL,
 			},
 		}
-		ca, cb := pipeWithVNet(v, a0TestConfig, a1TestConfig)
+		ca, cb := pipeWithVNet(t, vnet, a0TestConfig, a1TestConfig)
 
 		time.Sleep(1 * time.Second)
 
 		log.Debug("Closing...")
-		if !closePipe(t, ca, cb) {
-			return
-		}
+		closePipe(t, ca, cb)
 	})
 
 	t.Run("Symmetric NATs on both ends", func(t *testing.T) {
@@ -364,31 +387,27 @@ func TestConnectivityVNet(t *testing.T) {
 			MappingBehavior:   vnet.EndpointAddrPortDependent,
 			FilteringBehavior: vnet.EndpointAddrPortDependent,
 		}
-		v, err := buildVNet(natType, natType)
+		vnet, err := buildVNet(natType, natType)
 
-		if !assert.NoError(t, err, "should succeed") {
-			return
-		}
-		defer v.close()
+		require.NoError(t, err, "should succeed")
+		defer vnet.close()
 
 		log.Debug("Connecting...")
 		a0TestConfig := &agentTestConfig{
-			urls: []*URL{
+			urls: []*stun.URI{
 				stunServerURL,
 				turnServerURL,
 			},
 		}
 		a1TestConfig := &agentTestConfig{
-			urls: []*URL{
+			urls: []*stun.URI{
 				stunServerURL,
 			},
 		}
-		ca, cb := pipeWithVNet(v, a0TestConfig, a1TestConfig)
+		ca, cb := pipeWithVNet(t, vnet, a0TestConfig, a1TestConfig)
 
 		log.Debug("Closing...")
-		if !closePipe(t, ca, cb) {
-			return
-		}
+		closePipe(t, ca, cb)
 	})
 
 	t.Run("1:1 NAT with host candidate vs Symmetric NATs", func(t *testing.T) {
@@ -404,27 +423,23 @@ func TestConnectivityVNet(t *testing.T) {
 			MappingBehavior:   vnet.EndpointAddrPortDependent,
 			FilteringBehavior: vnet.EndpointAddrPortDependent,
 		}
-		v, err := buildVNet(natType0, natType1)
+		vnet, err := buildVNet(natType0, natType1)
 
-		if !assert.NoError(t, err, "should succeed") {
-			return
-		}
-		defer v.close()
+		require.NoError(t, err, "should succeed")
+		defer vnet.close()
 
 		log.Debug("Connecting...")
 		a0TestConfig := &agentTestConfig{
-			urls:                   []*URL{},
+			urls:                   []*stun.URI{},
 			nat1To1IPCandidateType: CandidateTypeHost, // Use 1:1 NAT IP as a host candidate
 		}
 		a1TestConfig := &agentTestConfig{
-			urls: []*URL{},
+			urls: []*stun.URI{},
 		}
-		ca, cb := pipeWithVNet(v, a0TestConfig, a1TestConfig)
+		ca, cb := pipeWithVNet(t, vnet, a0TestConfig, a1TestConfig)
 
 		log.Debug("Closing...")
-		if !closePipe(t, ca, cb) {
-			return
-		}
+		closePipe(t, ca, cb)
 	})
 
 	t.Run("1:1 NAT with srflx candidate vs Symmetric NATs", func(t *testing.T) {
@@ -440,37 +455,167 @@ func TestConnectivityVNet(t *testing.T) {
 			MappingBehavior:   vnet.EndpointAddrPortDependent,
 			FilteringBehavior: vnet.EndpointAddrPortDependent,
 		}
-		v, err := buildVNet(natType0, natType1)
+		vnet, err := buildVNet(natType0, natType1)
 
-		if !assert.NoError(t, err, "should succeed") {
-			return
-		}
-		defer v.close()
+		require.NoError(t, err, "should succeed")
+		defer vnet.close()
 
 		log.Debug("Connecting...")
 		a0TestConfig := &agentTestConfig{
-			urls:                   []*URL{},
+			urls:                   []*stun.URI{},
 			nat1To1IPCandidateType: CandidateTypeServerReflexive, // Use 1:1 NAT IP as a srflx candidate
 		}
 		a1TestConfig := &agentTestConfig{
-			urls: []*URL{},
+			urls: []*stun.URI{},
 		}
-		ca, cb := pipeWithVNet(v, a0TestConfig, a1TestConfig)
+		ca, cb := pipeWithVNet(t, vnet, a0TestConfig, a1TestConfig)
 
 		log.Debug("Closing...")
-		if !closePipe(t, ca, cb) {
-			return
-		}
+		closePipe(t, ca, cb)
 	})
 }
 
-// TestDisconnectedToConnected asserts that an agent can go to disconnected, and then return to connected successfully
-func TestDisconnectedToConnected(t *testing.T) {
-	report := test.CheckRoutines(t)
-	defer report()
+func TestConnectivityVNetWithAddressRewriteRuleOptions(t *testing.T) {
+	defer test.CheckRoutines(t)()
 
-	lim := test.TimeOut(time.Second * 10)
-	defer lim.Stop()
+	t.Run("host candidate mapping with options", func(t *testing.T) {
+		natType0 := &vnet.NATType{Mode: vnet.NATModeNAT1To1}
+		natType1 := &vnet.NATType{
+			MappingBehavior:   vnet.EndpointAddrPortDependent,
+			FilteringBehavior: vnet.EndpointAddrPortDependent,
+		}
+
+		vnet, err := buildVNet(natType0, natType1)
+		require.NoError(t, err)
+		defer vnet.close()
+
+		agent0Opts := []AgentOption{
+			WithNet(vnet.net0),
+			WithNetworkTypes(supportedNetworkTypes()),
+			WithMulticastDNSMode(MulticastDNSModeDisabled),
+			WithAddressRewriteRules(AddressRewriteRule{
+				External:        []string{vnetGlobalIPA},
+				AsCandidateType: CandidateTypeHost,
+			}),
+		}
+		agent1Opts := []AgentOption{
+			WithNet(vnet.net1),
+			WithNetworkTypes(supportedNetworkTypes()),
+			WithMulticastDNSMode(MulticastDNSModeDisabled),
+		}
+
+		ca, cb := pipeWithVNetUsingOptions(t, agent0Opts, agent1Opts)
+		closePipe(t, ca, cb)
+	})
+
+	t.Run("srflx candidate mapping with options", func(t *testing.T) {
+		natType0 := &vnet.NATType{Mode: vnet.NATModeNAT1To1}
+		natType1 := &vnet.NATType{
+			MappingBehavior:   vnet.EndpointAddrPortDependent,
+			FilteringBehavior: vnet.EndpointAddrPortDependent,
+		}
+
+		vnet, err := buildVNet(natType0, natType1)
+		require.NoError(t, err)
+		defer vnet.close()
+
+		agent0Opts := []AgentOption{
+			WithNet(vnet.net0),
+			WithNetworkTypes(supportedNetworkTypes()),
+			WithMulticastDNSMode(MulticastDNSModeDisabled),
+			WithAddressRewriteRules(AddressRewriteRule{
+				External:        []string{vnetGlobalIPA},
+				AsCandidateType: CandidateTypeServerReflexive,
+			}),
+		}
+		agent1Opts := []AgentOption{
+			WithNet(vnet.net1),
+			WithNetworkTypes(supportedNetworkTypes()),
+			WithMulticastDNSMode(MulticastDNSModeDisabled),
+		}
+
+		ca, cb := pipeWithVNetUsingOptions(t, agent0Opts, agent1Opts)
+		closePipe(t, ca, cb)
+	})
+}
+
+func TestConnectivityVNetNAT1To1SharedFoundation(t *testing.T) {
+	defer test.CheckRoutines(t)()
+
+	natType0 := &vnet.NATType{Mode: vnet.NATModeNAT1To1}
+	natType1 := &vnet.NATType{}
+
+	vnet, err := buildVNet(natType0, natType1)
+	require.NoError(t, err)
+	defer vnet.close()
+
+	agent, err := NewAgentWithOptions(
+		WithNet(vnet.net0),
+		WithNetworkTypes([]NetworkType{NetworkTypeUDP4}),
+		WithMulticastDNSMode(MulticastDNSModeDisabled),
+		WithAddressRewriteRules(
+			AddressRewriteRule{
+				External:        []string{vnetGlobalIPA},
+				AsCandidateType: CandidateTypeHost,
+			},
+			AddressRewriteRule{
+				External:        []string{vnetGlobalIPA},
+				AsCandidateType: CandidateTypeServerReflexive,
+			},
+		),
+	)
+	require.NoError(t, err)
+	defer func() {
+		require.NoError(t, agent.Close())
+	}()
+
+	require.NoError(t, agent.OnCandidate(func(Candidate) {}))
+	require.NoError(t, agent.GatherCandidates())
+
+	require.Eventually(t, func() bool {
+		state, stateErr := agent.GetGatheringState()
+		require.NoError(t, stateErr)
+
+		return state == GatheringStateComplete
+	}, time.Second, 10*time.Millisecond)
+
+	foundationSeen := make(map[string]struct{})
+	typeCount := make(map[CandidateType]int)
+
+	for _, candidates := range agent.localCandidates {
+		for _, cand := range candidates {
+			if cand.Address() != vnetGlobalIPA {
+				continue
+			}
+
+			if cand.Type() != CandidateTypeHost && cand.Type() != CandidateTypeServerReflexive {
+				continue
+			}
+
+			foundation := cand.Foundation()
+			_, dup := foundationSeen[foundation]
+			require.Falsef(t, dup, "duplicate foundation %s for %s candidate", foundation, cand.Type())
+			foundationSeen[foundation] = struct{}{}
+			typeCount[cand.Type()]++
+		}
+	}
+
+	require.Equal(t, 1, typeCount[CandidateTypeHost], "expected exactly one host candidate for %s", vnetGlobalIPA)
+	require.Equal(
+		t,
+		1,
+		typeCount[CandidateTypeServerReflexive],
+		"expected exactly one srflx candidate for %s",
+		vnetGlobalIPA,
+	)
+}
+
+// TestDisconnectedToConnected requires that an agent can go to disconnected,
+// and then return to connected successfully.
+func TestDisconnectedToConnected(t *testing.T) {
+	defer test.CheckRoutines(t)()
+
+	defer test.TimeOut(time.Second * 10).Stop()
 
 	loggerFactory := logging.NewDefaultLoggerFactory()
 
@@ -479,7 +624,7 @@ func TestDisconnectedToConnected(t *testing.T) {
 		CIDR:          "0.0.0.0/0",
 		LoggerFactory: loggerFactory,
 	})
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	var dropAllData uint64
 	wan.AddChunkFilter(func(vnet.Chunk) bool {
@@ -489,16 +634,16 @@ func TestDisconnectedToConnected(t *testing.T) {
 	net0, err := vnet.NewNet(&vnet.NetConfig{
 		StaticIPs: []string{"192.168.0.1"},
 	})
-	assert.NoError(t, err)
-	assert.NoError(t, wan.AddNet(net0))
+	require.NoError(t, err)
+	require.NoError(t, wan.AddNet(net0))
 
 	net1, err := vnet.NewNet(&vnet.NetConfig{
 		StaticIPs: []string{"192.168.0.2"},
 	})
-	assert.NoError(t, err)
-	assert.NoError(t, wan.AddNet(net1))
+	require.NoError(t, err)
+	require.NoError(t, wan.AddNet(net1))
 
-	assert.NoError(t, wan.Start())
+	require.NoError(t, wan.Start())
 
 	disconnectTimeout := time.Second
 	keepaliveInterval := time.Millisecond * 20
@@ -512,7 +657,10 @@ func TestDisconnectedToConnected(t *testing.T) {
 		KeepaliveInterval:   &keepaliveInterval,
 		CheckInterval:       &keepaliveInterval,
 	})
-	assert.NoError(t, err)
+	require.NoError(t, err)
+	defer func() {
+		require.NoError(t, controllingAgent.Close())
+	}()
 
 	controlledAgent, err := NewAgent(&AgentConfig{
 		NetworkTypes:        supportedNetworkTypes(),
@@ -522,19 +670,22 @@ func TestDisconnectedToConnected(t *testing.T) {
 		KeepaliveInterval:   &keepaliveInterval,
 		CheckInterval:       &keepaliveInterval,
 	})
-	assert.NoError(t, err)
+	require.NoError(t, err)
+	defer func() {
+		require.NoError(t, controlledAgent.Close())
+	}()
 
 	controllingStateChanges := make(chan ConnectionState, 100)
-	assert.NoError(t, controllingAgent.OnConnectionStateChange(func(c ConnectionState) {
+	require.NoError(t, controllingAgent.OnConnectionStateChange(func(c ConnectionState) {
 		controllingStateChanges <- c
 	}))
 
 	controlledStateChanges := make(chan ConnectionState, 100)
-	assert.NoError(t, controlledAgent.OnConnectionStateChange(func(c ConnectionState) {
+	require.NoError(t, controlledAgent.OnConnectionStateChange(func(c ConnectionState) {
 		controlledStateChanges <- c
 	}))
 
-	connectWithVNet(controllingAgent, controlledAgent)
+	connectWithVNet(t, controllingAgent, controlledAgent)
 	blockUntilStateSeen := func(expectedState ConnectionState, stateQueue chan ConnectionState) {
 		for s := range stateQueue {
 			if s == expectedState {
@@ -557,18 +708,14 @@ func TestDisconnectedToConnected(t *testing.T) {
 	blockUntilStateSeen(ConnectionStateConnected, controllingStateChanges)
 	blockUntilStateSeen(ConnectionStateConnected, controlledStateChanges)
 
-	assert.NoError(t, wan.Stop())
-	assert.NoError(t, controllingAgent.Close())
-	assert.NoError(t, controlledAgent.Close())
+	require.NoError(t, wan.Stop())
 }
 
-// Agent.Write should use the best valid pair if a selected pair is not yet available
+// Agent.Write should use the best valid pair if a selected pair is not yet available.
 func TestWriteUseValidPair(t *testing.T) {
-	report := test.CheckRoutines(t)
-	defer report()
+	defer test.CheckRoutines(t)()
 
-	lim := test.TimeOut(time.Second * 10)
-	defer lim.Stop()
+	defer test.TimeOut(time.Second * 10).Stop()
 
 	loggerFactory := logging.NewDefaultLoggerFactory()
 
@@ -577,7 +724,7 @@ func TestWriteUseValidPair(t *testing.T) {
 		CIDR:          "0.0.0.0/0",
 		LoggerFactory: loggerFactory,
 	})
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	wan.AddChunkFilter(func(c vnet.Chunk) bool {
 		if stun.IsMessage(c.UserData()) {
@@ -597,16 +744,16 @@ func TestWriteUseValidPair(t *testing.T) {
 	net0, err := vnet.NewNet(&vnet.NetConfig{
 		StaticIPs: []string{"192.168.0.1"},
 	})
-	assert.NoError(t, err)
-	assert.NoError(t, wan.AddNet(net0))
+	require.NoError(t, err)
+	require.NoError(t, wan.AddNet(net0))
 
 	net1, err := vnet.NewNet(&vnet.NetConfig{
 		StaticIPs: []string{"192.168.0.2"},
 	})
-	assert.NoError(t, err)
-	assert.NoError(t, wan.AddNet(net1))
+	require.NoError(t, err)
+	require.NoError(t, wan.AddNet(net1))
 
-	assert.NoError(t, wan.Start())
+	require.NoError(t, wan.Start())
 
 	// Create two agents and connect them
 	controllingAgent, err := NewAgent(&AgentConfig{
@@ -614,25 +761,31 @@ func TestWriteUseValidPair(t *testing.T) {
 		MulticastDNSMode: MulticastDNSModeDisabled,
 		Net:              net0,
 	})
-	assert.NoError(t, err)
+	require.NoError(t, err)
+	defer func() {
+		require.NoError(t, controllingAgent.Close())
+	}()
 
 	controlledAgent, err := NewAgent(&AgentConfig{
 		NetworkTypes:     supportedNetworkTypes(),
 		MulticastDNSMode: MulticastDNSModeDisabled,
 		Net:              net1,
 	})
-	assert.NoError(t, err)
+	require.NoError(t, err)
+	defer func() {
+		require.NoError(t, controlledAgent.Close())
+	}()
 
-	gatherAndExchangeCandidates(controllingAgent, controlledAgent)
+	gatherAndExchangeCandidates(t, controllingAgent, controlledAgent)
 
 	controllingUfrag, controllingPwd, err := controllingAgent.GetLocalUserCredentials()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	controlledUfrag, controlledPwd, err := controlledAgent.GetLocalUserCredentials()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
-	assert.NoError(t, controllingAgent.startConnectivityChecks(true, controlledUfrag, controlledPwd))
-	assert.NoError(t, controlledAgent.startConnectivityChecks(false, controllingUfrag, controllingPwd))
+	require.NoError(t, controllingAgent.startConnectivityChecks(true, controlledUfrag, controlledPwd))
+	require.NoError(t, controlledAgent.startConnectivityChecks(false, controllingUfrag, controllingPwd))
 
 	testMessage := []byte("Test Message")
 	go func() {
@@ -647,11 +800,9 @@ func TestWriteUseValidPair(t *testing.T) {
 
 	readBuf := make([]byte, len(testMessage))
 	_, err = (&Conn{agent: controlledAgent}).Read(readBuf)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
-	assert.Equal(t, readBuf, testMessage)
+	require.Equal(t, readBuf, testMessage)
 
-	assert.NoError(t, wan.Stop())
-	assert.NoError(t, controllingAgent.Close())
-	assert.NoError(t, controlledAgent.Close())
+	require.NoError(t, wan.Stop())
 }
