@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package ice
 
 import (
@@ -10,7 +13,7 @@ import (
 	"time"
 
 	"github.com/pion/logging"
-	"github.com/pion/transport/v2/packetio"
+	"github.com/pion/transport/v3/packetio"
 )
 
 type bufferedConn struct {
@@ -33,6 +36,7 @@ func newBufferedConn(conn net.Conn, bufSize int, logger logging.LeveledLogger) n
 	}
 
 	go bc.writeProcess()
+
 	return bc
 }
 
@@ -41,6 +45,7 @@ func (bc *bufferedConn) Write(b []byte) (int, error) {
 	if err != nil {
 		return n, err
 	}
+
 	return n, nil
 }
 
@@ -53,12 +58,14 @@ func (bc *bufferedConn) writeProcess() {
 		}
 
 		if err != nil {
-			bc.logger.Warnf("read buffer error: %s", err)
+			bc.logger.Warnf("Failed to read from buffer: %s", err)
+
 			continue
 		}
 
 		if _, err := bc.Conn.Write(pktBuf[:n]); err != nil {
-			bc.logger.Warnf("write error: %s", err)
+			bc.logger.Warnf("Failed to write: %s", err)
+
 			continue
 		}
 	}
@@ -67,6 +74,7 @@ func (bc *bufferedConn) writeProcess() {
 func (bc *bufferedConn) Close() error {
 	atomic.StoreInt32(&bc.closed, 1)
 	_ = bc.buf.Close()
+
 	return bc.Conn.Close()
 }
 
@@ -82,6 +90,7 @@ type tcpPacketConn struct {
 	wg         sync.WaitGroup
 	closedChan chan struct{}
 	closeOnce  sync.Once
+	aliveTimer *time.Timer
 }
 
 type streamingPacket struct {
@@ -91,14 +100,15 @@ type streamingPacket struct {
 }
 
 type tcpPacketParams struct {
-	ReadBuffer  int
-	LocalAddr   net.Addr
-	Logger      logging.LeveledLogger
-	WriteBuffer int
+	ReadBuffer    int
+	LocalAddr     net.Addr
+	Logger        logging.LeveledLogger
+	WriteBuffer   int
+	AliveDuration time.Duration
 }
 
 func newTCPPacketConn(params tcpPacketParams) *tcpPacketConn {
-	p := &tcpPacketConn{
+	packet := &tcpPacketConn{
 		params: &params,
 
 		conns: map[string]net.Conn{},
@@ -107,11 +117,31 @@ func newTCPPacketConn(params tcpPacketParams) *tcpPacketConn {
 		closedChan: make(chan struct{}),
 	}
 
-	return p
+	if params.AliveDuration > 0 {
+		packet.aliveTimer = time.AfterFunc(params.AliveDuration, func() {
+			packet.params.Logger.Warn("close tcp packet conn by alive timeout")
+			_ = packet.Close()
+		})
+	}
+
+	return packet
+}
+
+func (t *tcpPacketConn) ClearAliveTimer() {
+	t.mu.Lock()
+	if t.aliveTimer != nil {
+		t.aliveTimer.Stop()
+	}
+	t.mu.Unlock()
 }
 
 func (t *tcpPacketConn) AddConn(conn net.Conn, firstPacketData []byte) error {
-	t.params.Logger.Infof("AddConn: %s remote %s to local %s", conn.RemoteAddr().Network(), conn.RemoteAddr(), conn.LocalAddr())
+	t.params.Logger.Infof(
+		"Added connection: %s remote %s to local %s",
+		conn.RemoteAddr().Network(),
+		conn.RemoteAddr(),
+		conn.LocalAddr(),
+	)
 
 	t.mu.Lock()
 	defer t.mu.Unlock()
@@ -156,18 +186,20 @@ func (t *tcpPacketConn) startReading(conn net.Conn) {
 
 	for {
 		n, err := readStreamingPacket(conn, buf)
-		// t.params.Logger.Infof("readStreamingPacket read %d bytes", n)
 		if err != nil {
-			t.params.Logger.Infof("%v: %s", errReadingStreamingPacket, err)
-			t.handleRecv(streamingPacket{nil, conn.RemoteAddr(), err})
-			t.removeConn(conn)
+			t.params.Logger.Warnf("Failed to read streaming packet: %s", err)
+			last := t.removeConn(conn)
+			// Only propagate connection closure errors if no other open connection exists.
+			if last || (!errors.Is(err, io.EOF) && !errors.Is(err, net.ErrClosed)) {
+				t.handleRecv(streamingPacket{nil, conn.RemoteAddr(), err})
+			}
+
 			return
 		}
 
 		data := make([]byte, n)
 		copy(data, buf[:n])
 
-		// t.params.Logger.Infof("Writing read streaming packet to recvChan: %d bytes", len(data))
 		t.handleRecv(streamingPacket{data, conn.RemoteAddr(), nil})
 	}
 }
@@ -215,6 +247,7 @@ func (t *tcpPacketConn) ReadFrom(b []byte) (n int, rAddr net.Addr, err error) {
 
 	n = len(pkt.Data)
 	copy(b, pkt.Data[:n])
+
 	return n, pkt.RAddr, err
 }
 
@@ -226,20 +259,12 @@ func (t *tcpPacketConn) WriteTo(buf []byte, rAddr net.Addr) (n int, err error) {
 
 	if !ok {
 		return 0, io.ErrClosedPipe
-		// conn, err := net.DialTCP(tcp, nil, rAddr.(*net.TCPAddr))
-
-		// if err != nil {
-		// 	t.params.Logger.Tracef("DialTCP error: %s", err)
-		// 	return 0, err
-		// }
-
-		// go t.startReading(conn)
-		// t.conns[rAddr.String()] = conn
 	}
 
 	n, err = writeStreamingPacket(conn, buf)
 	if err != nil {
-		t.params.Logger.Tracef("%w %s", errWriting, rAddr)
+		t.params.Logger.Tracef("%w %s", errWrite, rAddr)
+
 		return n, err
 	}
 
@@ -253,13 +278,20 @@ func (t *tcpPacketConn) closeAndLogError(closer io.Closer) {
 	}
 }
 
-func (t *tcpPacketConn) removeConn(conn net.Conn) {
+func (t *tcpPacketConn) removeConn(conn net.Conn) bool {
 	t.mu.Lock()
 	defer t.mu.Unlock()
 
 	t.closeAndLogError(conn)
 
+	// wait for some time to flush pending writes
+	_ = conn.SetWriteDeadline(time.Now().Add(5 * time.Second))
+	// read deadline as well just in case
+	_ = conn.SetReadDeadline(time.Now().Add(5 * time.Second))
+
 	delete(t.conns, conn.RemoteAddr().String())
+
+	return len(t.conns) == 0
 }
 
 func (t *tcpPacketConn) Close() error {
@@ -269,10 +301,19 @@ func (t *tcpPacketConn) Close() error {
 	t.closeOnce.Do(func() {
 		close(t.closedChan)
 		shouldCloseRecvChan = true
+		if t.aliveTimer != nil {
+			t.aliveTimer.Stop()
+		}
 	})
 
 	for _, conn := range t.conns {
 		t.closeAndLogError(conn)
+
+		// wait for some time to flush pending writes
+		_ = conn.SetWriteDeadline(time.Now().Add(5 * time.Second))
+		// read deadline as well just in case
+		_ = conn.SetReadDeadline(time.Now().Add(5 * time.Second))
+
 		delete(t.conns, conn.RemoteAddr().String())
 	}
 
@@ -291,15 +332,15 @@ func (t *tcpPacketConn) LocalAddr() net.Addr {
 	return t.params.LocalAddr
 }
 
-func (t *tcpPacketConn) SetDeadline(tm time.Time) error {
+func (t *tcpPacketConn) SetDeadline(time.Time) error {
 	return nil
 }
 
-func (t *tcpPacketConn) SetReadDeadline(tm time.Time) error {
+func (t *tcpPacketConn) SetReadDeadline(time.Time) error {
 	return nil
 }
 
-func (t *tcpPacketConn) SetWriteDeadline(tm time.Time) error {
+func (t *tcpPacketConn) SetWriteDeadline(time.Time) error {
 	return nil
 }
 
