@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package ice
 
 import (
@@ -6,7 +9,7 @@ import (
 	"sync/atomic"
 	"time"
 
-	"github.com/pion/stun"
+	"github.com/pion/stun/v3"
 )
 
 // Dial connects to the remote agent, acting as the controlling ice agent.
@@ -24,23 +27,23 @@ func (a *Agent) Accept(ctx context.Context, remoteUfrag, remotePwd string) (*Con
 // Conn represents the ICE connection.
 // At the moment the lifetime of the Conn is equal to the Agent.
 type Conn struct {
-	bytesReceived uint64
-	bytesSent     uint64
+	bytesReceived atomic.Uint64
+	bytesSent     atomic.Uint64
 	agent         *Agent
 }
 
-// BytesSent returns the number of bytes sent
+// BytesSent returns the number of bytes sent.
 func (c *Conn) BytesSent() uint64 {
-	return atomic.LoadUint64(&c.bytesSent)
+	return c.bytesSent.Load()
 }
 
-// BytesReceived returns the number of bytes received
+// BytesReceived returns the number of bytes received.
 func (c *Conn) BytesReceived() uint64 {
-	return atomic.LoadUint64(&c.bytesReceived)
+	return c.bytesReceived.Load()
 }
 
 func (a *Agent) connect(ctx context.Context, isControlling bool, remoteUfrag, remotePwd string) (*Conn, error) {
-	err := a.ok()
+	err := a.loop.Err()
 	if err != nil {
 		return nil, err
 	}
@@ -49,10 +52,10 @@ func (a *Agent) connect(ctx context.Context, isControlling bool, remoteUfrag, re
 		return nil, err
 	}
 
-	// block until pair selected
+	// Block until pair selected
 	select {
-	case <-a.done:
-		return nil, a.getErr()
+	case <-a.loop.Done():
+		return nil, a.loop.Err()
 	case <-ctx.Done():
 		return nil, ErrCanceledByCaller
 	case <-a.onConnected:
@@ -65,31 +68,32 @@ func (a *Agent) connect(ctx context.Context, isControlling bool, remoteUfrag, re
 
 // Read implements the Conn Read method.
 func (c *Conn) Read(p []byte) (int, error) {
-	err := c.agent.ok()
+	err := c.agent.loop.Err()
 	if err != nil {
 		return 0, err
 	}
 
 	n, err := c.agent.buf.Read(p)
-	atomic.AddUint64(&c.bytesReceived, uint64(n))
+	c.bytesReceived.Add(uint64(n)) //nolint:gosec // G115
+
 	return n, err
 }
 
 // Write implements the Conn Write method.
-func (c *Conn) Write(p []byte) (int, error) {
-	err := c.agent.ok()
+func (c *Conn) Write(packet []byte) (int, error) {
+	err := c.agent.loop.Err()
 	if err != nil {
 		return 0, err
 	}
 
-	if stun.IsMessage(p) {
-		return 0, errICEWriteSTUNMessage
+	if stun.IsMessage(packet) {
+		return 0, errWriteSTUNMessageToIceConn
 	}
 
 	pair := c.agent.getSelectedPair()
 	if pair == nil {
-		if err = c.agent.run(c.agent.context(), func(ctx context.Context, a *Agent) {
-			pair = a.getBestValidCandidatePair()
+		if err = c.agent.loop.Run(c.agent.loop, func(_ context.Context) {
+			pair = c.agent.getBestValidCandidatePair()
 		}); err != nil {
 			return 0, err
 		}
@@ -99,8 +103,14 @@ func (c *Conn) Write(p []byte) (int, error) {
 		}
 	}
 
-	atomic.AddUint64(&c.bytesSent, uint64(len(p)))
-	return pair.Write(p)
+	// Write application data via the selected pair and update stats with actual bytes written.
+	n, err := pair.Write(packet)
+	if n > 0 {
+		c.bytesSent.Add(uint64(n))
+		pair.UpdatePacketSent(n)
+	}
+
+	return n, err
 }
 
 // Close implements the Conn Close method. It is used to close
@@ -129,17 +139,17 @@ func (c *Conn) RemoteAddr() net.Addr {
 	return pair.Remote.addr()
 }
 
-// SetDeadline is a stub
-func (c *Conn) SetDeadline(t time.Time) error {
+// SetDeadline is a stub.
+func (c *Conn) SetDeadline(time.Time) error {
 	return nil
 }
 
-// SetReadDeadline is a stub
-func (c *Conn) SetReadDeadline(t time.Time) error {
+// SetReadDeadline is a stub.
+func (c *Conn) SetReadDeadline(time.Time) error {
 	return nil
 }
 
-// SetWriteDeadline is a stub
-func (c *Conn) SetWriteDeadline(t time.Time) error {
+// SetWriteDeadline is a stub.
+func (c *Conn) SetWriteDeadline(time.Time) error {
 	return nil
 }
