@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 //go:build !js
 // +build !js
 
@@ -9,58 +12,56 @@ import (
 	"testing"
 	"time"
 
-	"github.com/pion/transport/v2/test"
-	"github.com/pion/transport/v2/vnet"
-	"github.com/stretchr/testify/assert"
+	"github.com/pion/stun/v3"
+	"github.com/pion/transport/v3/test"
+	"github.com/pion/transport/v3/vnet"
+	"github.com/stretchr/testify/require"
 )
 
 func TestRemoteLocalAddr(t *testing.T) {
 	// Check for leaking routines
-	report := test.CheckRoutines(t)
-	defer report()
+	defer test.CheckRoutines(t)()
 
 	// Limit runtime in case of deadlocks
-	lim := test.TimeOut(time.Second * 20)
-	defer lim.Stop()
+	defer test.TimeOut(time.Second * 20).Stop()
 
 	// Agent0 is behind 1:1 NAT
 	natType0 := &vnet.NATType{Mode: vnet.NATModeNAT1To1}
 	// Agent1 is behind 1:1 NAT
 	natType1 := &vnet.NATType{Mode: vnet.NATModeNAT1To1}
 
-	v, errVnet := buildVNet(natType0, natType1)
-	if !assert.NoError(t, errVnet, "should succeed") {
-		return
-	}
-	defer v.close()
+	builtVnet, errVnet := buildVNet(natType0, natType1)
+	require.NoError(t, errVnet, "should succeed")
+	defer builtVnet.close()
 
-	stunServerURL := &URL{
-		Scheme: SchemeTypeSTUN,
+	stunServerURL := &stun.URI{
+		Scheme: stun.SchemeTypeSTUN,
 		Host:   vnetSTUNServerIP,
 		Port:   vnetSTUNServerPort,
-		Proto:  ProtoTypeUDP,
+		Proto:  stun.ProtoTypeUDP,
 	}
 
 	t.Run("Disconnected Returns nil", func(t *testing.T) {
 		disconnectedAgent, err := NewAgent(&AgentConfig{})
-		assert.NoError(t, err)
+		require.NoError(t, err)
 
 		disconnectedConn := Conn{agent: disconnectedAgent}
-		assert.Nil(t, disconnectedConn.RemoteAddr())
-		assert.Nil(t, disconnectedConn.LocalAddr())
+		require.Nil(t, disconnectedConn.RemoteAddr())
+		require.Nil(t, disconnectedConn.LocalAddr())
 
-		assert.NoError(t, disconnectedConn.Close())
+		require.NoError(t, disconnectedConn.Close())
 	})
 
 	t.Run("Remote/Local Pair Match between Agents", func(t *testing.T) {
-		ca, cb := pipeWithVNet(v,
+		ca, cb := pipeWithVNet(t, builtVnet,
 			&agentTestConfig{
-				urls: []*URL{stunServerURL},
+				urls: []*stun.URI{stunServerURL},
 			},
 			&agentTestConfig{
-				urls: []*URL{stunServerURL},
+				urls: []*stun.URI{stunServerURL},
 			},
 		)
+		defer closePipe(t, ca, cb)
 
 		aRAddr := ca.RemoteAddr()
 		aLAddr := ca.LocalAddr()
@@ -68,27 +69,23 @@ func TestRemoteLocalAddr(t *testing.T) {
 		bLAddr := cb.LocalAddr()
 
 		// Assert that nothing is nil
-		assert.NotNil(t, aRAddr)
-		assert.NotNil(t, aLAddr)
-		assert.NotNil(t, bRAddr)
-		assert.NotNil(t, bLAddr)
+		require.NotNil(t, aRAddr)
+		require.NotNil(t, aLAddr)
+		require.NotNil(t, bRAddr)
+		require.NotNil(t, bLAddr)
 
 		// Assert addresses
-		assert.Equal(t, aLAddr.String(),
+		require.Equal(t, aLAddr.String(),
 			fmt.Sprintf("%s:%d", vnetLocalIPA, bRAddr.(*net.UDPAddr).Port), //nolint:forcetypeassert
 		)
-		assert.Equal(t, bLAddr.String(),
+		require.Equal(t, bLAddr.String(),
 			fmt.Sprintf("%s:%d", vnetLocalIPB, aRAddr.(*net.UDPAddr).Port), //nolint:forcetypeassert
 		)
-		assert.Equal(t, aRAddr.String(),
+		require.Equal(t, aRAddr.String(),
 			fmt.Sprintf("%s:%d", vnetGlobalIPB, bLAddr.(*net.UDPAddr).Port), //nolint:forcetypeassert
 		)
-		assert.Equal(t, bRAddr.String(),
+		require.Equal(t, bRAddr.String(),
 			fmt.Sprintf("%s:%d", vnetGlobalIPA, aLAddr.(*net.UDPAddr).Port), //nolint:forcetypeassert
 		)
-
-		// Close
-		assert.NoError(t, ca.Close())
-		assert.NoError(t, cb.Close())
 	})
 }
